import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import fr.cnes.dynvol.psimu.calc.output.INewVarsFunction;
import fr.cnes.dynvol.psimu.calc.utils.PsimuUtils;
import fr.cnes.genius.exception.SqliteException;
import fr.cnes.genius.sqlite.ColumnInfo.ColumnType;
import fr.cnes.genius.sqlite.ResultWriter;
import fr.cnes.sirius.patrius.bodies.OneAxisEllipsoid;
import fr.cnes.sirius.patrius.propagation.SpacecraftState;

/**
 * Class used by PsimuCustomVariables tutorila and defining new variables (HPM and HAM resp. mean perigee and apogee altitudes).
 * @author goesterjf
 *
 */
public class MyNewMeanVars implements INewVarsFunction {
    
    private final OneAxisEllipsoid earth;
    private final ArrayList<String> listOfNames;
    
    public MyNewMeanVars ( final OneAxisEllipsoid earth ) {
        
       this.earth = earth;
       // List initialization
       listOfNames = new ArrayList<String>();
       listOfNames.add("HPM");
       listOfNames.add("HAM");

    }
    
    @Override
    public void addNewVars ( final ResultWriter resultWriter ) throws SqliteException {

        if ( resultWriter != null) {

            // Mean perigee altitude creation
            String varName = listOfNames.get(0);
            String varDesc = "Mean perigee altitude";
            String varUnit = "km";
            boolean varVisibility = true;
            
            // Adding a column in the ephemeris table
            resultWriter.addColumn(PsimuUtils.EPH_TABLE, varName, varDesc,
                    ColumnType.REAL, varUnit, null, varVisibility);
    
            // Adding a column in the event table
            resultWriter.addColumn(PsimuUtils.EVENT_TABLE, varName,
                    varDesc, ColumnType.REAL, varUnit, null, varVisibility);
    
            // Mean apogee altitude creation
            varName = listOfNames.get(1);
            varDesc = "Mean apogee altitude";
            
            // Adding a column in the ephemeris table
            resultWriter.addColumn(PsimuUtils.EPH_TABLE, varName, varDesc,
                    ColumnType.REAL, varUnit, null, varVisibility);
    
            // Adding a column in the event table
            resultWriter.addColumn(PsimuUtils.EVENT_TABLE, varName,
                    varDesc, ColumnType.REAL, varUnit, null, varVisibility);
        
        }

    }

    @Override
    public void computeNewVars(final ResultWriter resultWriter, final String tableName,
            final SpacecraftState currentState,
            final Map<String, Object> listOfCurrentVars) throws SqliteException {
        
        // Getting previously computed mean semi major axis and eccentricity
        final double am = (Double)listOfCurrentVars.get("AM");
        final double em = (Double)listOfCurrentVars.get("EM");
        // Computation of the mean perigee and apogee altitude
        final double hpm = am*(1. - em) - earth.getEquatorialRadius();
        final double ham = am*(1. + em) - earth.getEquatorialRadius();
        
        // Storing in the table
        if ( resultWriter != null) {
            resultWriter.addValue(tableName, listOfNames.get(0), hpm);
            resultWriter.addValue(tableName, listOfNames.get(1), ham);
        }
        
        // Storing in the current list
        if ( listOfCurrentVars != null ) {
            listOfCurrentVars.put("HPM", hpm);
            listOfCurrentVars.put("HAM", ham);
        }
        
    }

    @Override
    public List<String> getNames() {
        
        return listOfNames;
        
    }

}
