import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import fr.cnes.dynvol.psimu.calc.integrator.PsimuPropagationData;
import fr.cnes.dynvol.psimu.calc.integrator.firstOrderIntegrators.RK4data;
import fr.cnes.dynvol.psimu.calc.main.Psimu;
import fr.cnes.dynvol.psimu.calc.main.Psimu.StorageType;
import fr.cnes.dynvol.psimu.calc.main.UserFramesConfsEnum;
import fr.cnes.dynvol.psimu.calc.output.OutputConfig;
import fr.cnes.dynvol.psimu.exception.PsimuException;
import fr.cnes.dynvol.psimu.exception.PsimuOutputManagerException;
import fr.cnes.dynvol.psimu.exception.PsimuPropagationException;
import fr.cnes.genopus.nonWidget.events.CustomRecurrentDateDetector;
import fr.cnes.genopus.nonWidget.exception.NonWidgetException;
import fr.cnes.genopus.nonWidget.maneuvers.CustomConstantManeuver;
import fr.cnes.genopus.nonWidget.maneuvers.CustomImpulseDaManeuver;
import fr.cnes.genopus.nonWidget.maneuvers.CustomManeuverSequence;
import fr.cnes.sirius.addons.patriusdataset.PatriusDataset;
import fr.cnes.sirius.patrius.assembly.Assembly;
import fr.cnes.sirius.patrius.assembly.Vehicle;
import fr.cnes.sirius.patrius.assembly.models.MassModel;
import fr.cnes.sirius.patrius.assembly.properties.MassProperty;
import fr.cnes.sirius.patrius.assembly.properties.PropulsiveProperty;
import fr.cnes.sirius.patrius.assembly.properties.TankProperty;
import fr.cnes.sirius.patrius.assembly.vehicle.VehicleSurfaceModel;
import fr.cnes.sirius.patrius.attitudes.AttitudeLaw;
import fr.cnes.sirius.patrius.attitudes.LofOffset;
import fr.cnes.sirius.patrius.bodies.OneAxisEllipsoid;
import fr.cnes.sirius.patrius.forces.ForceModel;
import fr.cnes.sirius.patrius.forces.ForceModelsData;
import fr.cnes.sirius.patrius.forces.gravity.EarthGravitationalModelFactory;
import fr.cnes.sirius.patrius.forces.gravity.EarthGravitationalModelFactory.GravityFieldNames;
import fr.cnes.sirius.patrius.frames.Frame;
import fr.cnes.sirius.patrius.frames.FramesFactory;
import fr.cnes.sirius.patrius.frames.LOFType;
import fr.cnes.sirius.patrius.math.geometry.euclidean.threed.Sphere;
import fr.cnes.sirius.patrius.math.geometry.euclidean.threed.Vector3D;
import fr.cnes.sirius.patrius.math.util.FastMath;
import fr.cnes.sirius.patrius.orbits.KeplerianOrbit;
import fr.cnes.sirius.patrius.orbits.PositionAngle;
import fr.cnes.sirius.patrius.propagation.MassProvider;
import fr.cnes.sirius.patrius.propagation.events.AbstractDetector;
import fr.cnes.sirius.patrius.propagation.events.EventDetector.Action;
import fr.cnes.sirius.patrius.time.AbsoluteDate;
import fr.cnes.sirius.patrius.time.TimeScale;
import fr.cnes.sirius.patrius.time.TimeScalesFactory;
import fr.cnes.sirius.patrius.utils.Constants;
import fr.cnes.sirius.patrius.utils.exception.PatriusException;

/**
 * Tutorial to use PSIMU with with maneuvers defined with recurrent dates.
 * @author goesterjf
 *
 */
public class PsimuWithManeuversWithRecurrentDates {

    public static void main(String[] args) throws PatriusException, NonWidgetException, IOException, ParseException, PsimuException, PsimuPropagationException, PsimuOutputManagerException {
        
        // Patrius Dataset initialization (needed for example to get the UTC time)
        PatriusDataset.addResourcesFromPatriusDataset() ;

        // Recovery of the UTC time scale using a "factory"
        final TimeScale TUC = TimeScalesFactory.getUTC();
        
        // Reference frames
        final Frame GCRF = FramesFactory.getGCRF();
        final Frame ITRF = FramesFactory.getITRF();

        // Constants that will be used later
        final double MU   = Constants.WGS84_EARTH_MU;
        final double REQ  = Constants.WGS84_EARTH_EQUATORIAL_RADIUS;
        final double FLAT = Constants.WGS84_EARTH_FLATTENING;
        final OneAxisEllipsoid EARTH = new OneAxisEllipsoid(REQ, FLAT, ITRF, "EARTH");
        
        // Orbital events tuning
        final double MAXCHECK = AbstractDetector.DEFAULT_MAXCHECK;
        final double THRESHOLD = AbstractDetector.DEFAULT_THRESHOLD;
        
        // ORBIT
        
        final AbsoluteDate date = new AbsoluteDate("2010-01-01T12:00:00.000", TUC);
        
        final double sma = REQ + 250.e3;
        final double ecc = 0.;
        final double inc = FastMath.toRadians(51.6);
        final double raan = FastMath.toRadians(0.);
        final double pa  = FastMath.toRadians(0.);
        final double ano = FastMath.toRadians(0.);
        final KeplerianOrbit iniOrbit = new KeplerianOrbit(sma, ecc, inc, raan, pa, ano, PositionAngle.MEAN, GCRF, date, MU);
        
        // VEHICLE
        
        // Dry mass
        final double dryMass = 1000.;
        final MassProperty dryMassProperty = new MassProperty(dryMass);

        // Propulsive properties
        final PropulsiveProperty engine = new PropulsiveProperty(400, 320.);
        engine.setPartName("ENGINE");
        final ArrayList<PropulsiveProperty> listOfEngines = new ArrayList<PropulsiveProperty>();
        listOfEngines.add(engine);
        final TankProperty tank = new TankProperty(200.);
        tank.setPartName("TANK");
        final ArrayList<TankProperty> listOfTanks = new ArrayList<TankProperty>();
        listOfTanks.add(tank);
        
        // Shape
        VehicleSurfaceModel vehicleRefSurface = new VehicleSurfaceModel(new Sphere(0.));

        final Vehicle vehicle = new Vehicle(vehicleRefSurface, null, dryMassProperty, null, null, listOfEngines, listOfTanks);

        final Assembly assembly = vehicle.createAssembly(GCRF);
        final MassProvider mm = new MassModel(assembly);
        
        // FORCES
        
        // Potential
        final ForceModel potential = EarthGravitationalModelFactory.getBalmino(GravityFieldNames.GRGS, "grim4s4_gr", 0, 0, true);
        
        final ForceModelsData forces = new ForceModelsData(potential, null, null, null, null, null, null, null, null, null, null, null, null,  null, null);
        
        // MANEUVERS
        
        // Impulse maneuver (Da = -1 km every period step)
        final double period = iniOrbit.getKeplerianPeriod();
        final AbsoluteDate startDate = date.shiftedBy(10.);
        final double da = 1000.;
        final CustomRecurrentDateDetector startEvent = new CustomRecurrentDateDetector(startDate, period, MAXCHECK, THRESHOLD, Action.STOP);
        final CustomImpulseDaManeuver imp =
                new CustomImpulseDaManeuver("Impulse maneuver", listOfEngines.get(0), listOfTanks.get(0), LOFType.TNW, startEvent, da, mm);
        
        // Constant thrust maneuver (thrust of 10s beginning after 1/2 period and with 1 period step)
        final double thrustDuration = 10.;
        final AbsoluteDate startDateCont = date.shiftedBy(0.5*period);
        final CustomRecurrentDateDetector startEventCont = new CustomRecurrentDateDetector(startDateCont, period, MAXCHECK, THRESHOLD, Action.STOP);
        final AbsoluteDate endDate = startDateCont.shiftedBy(thrustDuration);
        final CustomRecurrentDateDetector stopEventCont = new CustomRecurrentDateDetector(endDate, period, MAXCHECK, THRESHOLD, Action.STOP);
        final Vector3D direction = new Vector3D(-1., 0., 0.);
        CustomConstantManeuver cont = new CustomConstantManeuver("Continuous maneuver", listOfEngines.get(0), listOfTanks.get(0), LOFType.TNW, startEventCont, stopEventCont, direction, mm);
        
        final CustomManeuverSequence manSeq = new CustomManeuverSequence(0., 0.);
        manSeq.addManeuver(imp);
        manSeq.addManeuver(cont);
        
        // ATTITUDE
        
        final AttitudeLaw attitudeLaw = new LofOffset(LOFType.TNW);
        
        // INTEGRATION DATA

        final double duration = 5.*period;
        final double hStop = 120.e+3;
        final PsimuPropagationData propagationData = new PsimuPropagationData(duration, GCRF, hStop);
        
        final double step = 5.;
        final RK4data integrationData = new RK4data(propagationData, step);

        final Psimu test = new Psimu(iniOrbit, EARTH, UserFramesConfsEnum.ONLY_PREC_NUT, integrationData, vehicle, mm, forces, manSeq, attitudeLaw);
        
        // PROPAGATION
        
        final double outputStep = 600.;
        final OutputConfig output = new OutputConfig(FramesFactory.getEME2000(), LOFType.LVLH, outputStep, TimeScalesFactory.getUTC());        
        test.propagateInMasterMode(output, null, StorageType.MEMORY_ALL, StorageType.NONE);
      
        List<Map<String, Object>> listOfVars = test.getVariablesWithoutEventsList();
      
        for (int i = 0; i < listOfVars.size(); i++) {
            final double deltaA = sma - (Double)listOfVars.get(i).get("A");
            System.out.println(String.format(Locale.US, "Date: %s ; DeltaA = %8.3f km", (String)listOfVars.get(i).get("DATE"), deltaA/1000.));
        }

    }
    
}
