import java.io.IOException;
import java.text.ParseException;

import fr.cnes.dynvol.psimu.calc.integrator.PsimuPropagationData;
import fr.cnes.dynvol.psimu.calc.integrator.firstOrderIntegrators.DopData;
import fr.cnes.dynvol.psimu.calc.main.Psimu;
import fr.cnes.dynvol.psimu.calc.main.UserFramesConfsEnum;
import fr.cnes.dynvol.psimu.exception.PsimuException;
import fr.cnes.dynvol.psimu.exception.PsimuPropagationException;
import fr.cnes.genopus.nonWidget.exception.NonWidgetException;
import fr.cnes.sirius.addons.patriusdataset.PatriusDataset;
import fr.cnes.sirius.patrius.assembly.Assembly;
import fr.cnes.sirius.patrius.assembly.Vehicle;
import fr.cnes.sirius.patrius.assembly.models.MassModel;
import fr.cnes.sirius.patrius.assembly.models.RediffusedRadiativeModel;
import fr.cnes.sirius.patrius.assembly.properties.MassProperty;
import fr.cnes.sirius.patrius.assembly.properties.RadiativeIRProperty;
import fr.cnes.sirius.patrius.assembly.properties.RadiativeProperty;
import fr.cnes.sirius.patrius.assembly.vehicle.RadiativeProperties;
import fr.cnes.sirius.patrius.assembly.vehicle.VehicleSurfaceModel;
import fr.cnes.sirius.patrius.attitudes.AttitudeLaw;
import fr.cnes.sirius.patrius.attitudes.LofOffset;
import fr.cnes.sirius.patrius.bodies.CelestialBody;
import fr.cnes.sirius.patrius.bodies.MeeusSun;
import fr.cnes.sirius.patrius.bodies.OneAxisEllipsoid;
import fr.cnes.sirius.patrius.forces.ForceModel;
import fr.cnes.sirius.patrius.forces.ForceModelsData;
import fr.cnes.sirius.patrius.forces.gravity.EarthGravitationalModelFactory;
import fr.cnes.sirius.patrius.forces.gravity.EarthGravitationalModelFactory.GravityFieldNames;
import fr.cnes.sirius.patrius.forces.radiation.IEmissivityModel;
import fr.cnes.sirius.patrius.forces.radiation.KnockeRiesModel;
import fr.cnes.sirius.patrius.forces.radiation.RediffusedRadiationPressure;
import fr.cnes.sirius.patrius.forces.radiation.SolarRadiationPressure;
import fr.cnes.sirius.patrius.frames.Frame;
import fr.cnes.sirius.patrius.frames.FramesFactory;
import fr.cnes.sirius.patrius.frames.LOFType;
import fr.cnes.sirius.patrius.math.geometry.euclidean.threed.Sphere;
import fr.cnes.sirius.patrius.math.util.FastMath;
import fr.cnes.sirius.patrius.orbits.ApsisOrbit;
import fr.cnes.sirius.patrius.orbits.PositionAngle;
import fr.cnes.sirius.patrius.propagation.MassProvider;
import fr.cnes.sirius.patrius.propagation.SpacecraftState;
import fr.cnes.sirius.patrius.time.AbsoluteDate;
import fr.cnes.sirius.patrius.time.TimeScale;
import fr.cnes.sirius.patrius.time.TimeScalesFactory;
import fr.cnes.sirius.patrius.utils.Constants;
import fr.cnes.sirius.patrius.utils.exception.PatriusException;

/**
 * Tutorial to use PSIMU with with radiative pressure forces.
 * @author goesterjf
 *
 */
public class PsimuWithRadiativePressure {

    public static void main(String[] args) throws PatriusException, NonWidgetException, IOException, ParseException, PsimuException, PsimuPropagationException {
        
        // Patrius Dataset initialization
        PatriusDataset.addResourcesFromPatriusDataset() ;

        // Recovery of the UTC time scale using a "factory"
        final TimeScale TUC = TimeScalesFactory.getUTC();
        
        // Reference frames
        final Frame GCRF = FramesFactory.getGCRF();
        final Frame ITRF = FramesFactory.getITRF();

        // Constants that will be used later
        final double MU   = Constants.WGS84_EARTH_MU;
        final double REQ  = Constants.WGS84_EARTH_EQUATORIAL_RADIUS;
        final double FLAT = Constants.WGS84_EARTH_FLATTENING;
        final OneAxisEllipsoid EARTH = new OneAxisEllipsoid(REQ, FLAT, ITRF, "EARTH");
        
        // ORBIT
        
        final AbsoluteDate date = new AbsoluteDate("2010-01-01T12:00:00.000", TUC);
        
        final double rapo = REQ + 25000.e3;
        final double rper = REQ + 500.e3;
        final double inc = FastMath.toRadians(51.6);
        final double raan = FastMath.toRadians(0.);
        final double pa  = FastMath.toRadians(0.);
        final double ano = FastMath.toRadians(0.);
        final ApsisOrbit iniOrbit = new ApsisOrbit(rper, rapo, inc, pa, raan, ano, PositionAngle.MEAN, GCRF, date, MU);
        
        // VEHICLE
        
        // Dry mass
        final double dryMass = 1000.;
        final MassProperty dryMassProperty = new MassProperty(dryMass);
        
        // Shape
        final double sref = 5.0;
        Sphere sphere = new Sphere(sref);
        VehicleSurfaceModel vehicleRefSurface = new VehicleSurfaceModel(sphere);

        // Radiative properties
        final RadiativeProperty prop1 = new RadiativeProperty(1., 0., 0.);
        final RadiativeIRProperty prop2 = new RadiativeIRProperty(1., 0., 0.);
        final RadiativeProperties radProp = new RadiativeProperties(prop1, prop2, vehicleRefSurface);
        
        final Vehicle vehicle = new Vehicle(vehicleRefSurface, null, dryMassProperty, null, radProp, null, null);

        final Assembly assembly = vehicle.createAssembly(GCRF);
        final MassProvider mm = new MassModel(assembly);
        
        // FORCES
        
        // Potential
        final ForceModel potential = EarthGravitationalModelFactory.getDroziner(GravityFieldNames.GRGS, "grim4s4_gr", 8, 8, true);
        
        // Radiative pressure
        final CelestialBody sunBody = new MeeusSun();
        
        // By default values used when GUI mode ...
        
        // Direct SRP
        final double dRef = 1.4959787E11;
        final double pRef = 4.5605E-6;
        SolarRadiationPressure radPres = new SolarRadiationPressure(dRef, pRef, sunBody, EARTH, assembly, 1.);
       
        // Rediffused SRP data
        final int inCorona = 1;
        final int inMeridian = 10;
        final IEmissivityModel inEmissivityModel = new KnockeRiesModel();
        final boolean inAlbedo = false;
        final boolean inIr = false;
        final double coefAlbedo = 0.;
        final double coefIr = 0.;
        RediffusedRadiativeModel rdm = new RediffusedRadiativeModel(inAlbedo, inIr, coefAlbedo, coefIr, assembly);
        RediffusedRadiationPressure reDiff =  new RediffusedRadiationPressure(sunBody, GCRF, inCorona, inMeridian, inEmissivityModel, rdm);
        
        final ForceModelsData forces = new ForceModelsData(potential, null, null, null, null, null, null, null, null, null, null, null, null,  null, null);
        forces.setSolarRadiationPressure(radPres);
        forces.setRediffusedRadiationPressure(reDiff);
        
        // ATTITUDE
        
        final AttitudeLaw attitudeLaw = new LofOffset(LOFType.TNW);
        
        // INTEGRATION DATA

        final double duration = 86400.;
        final double hStop = 0.e+3;
        final PsimuPropagationData propagationData = new PsimuPropagationData(duration, GCRF, hStop);
        
        final double minStep = 0.1;
        final double maxStep = 30.;
        final double[] vecAbsoluteTolerance = { 1.0E-5, 1.0E-5, 1.0E-5, 1.0E-8, 1.0E-8, 1.0E-8 };
        final double[] vecRelativeTolerance = { 1.0E-8, 1.0E-8, 1.0E-8, 1.0E-8, 1.0E-8, 1.0E-8 };
        final double absMassTolerance = 1.0E-3;
        final double relMassTolerance = 1.0E-2;
        final DopData integrationData = new DopData(propagationData, minStep, maxStep, vecAbsoluteTolerance, vecRelativeTolerance, absMassTolerance, relMassTolerance);
        
        // PSIMU initialization
        
        final Psimu test = new Psimu(iniOrbit, EARTH, UserFramesConfsEnum.ONLY_PREC_NUT, integrationData, vehicle, mm, forces, null, attitudeLaw);
        
        System.out.println("Initial orbit:");
        System.out.println("Date:"+iniOrbit.getDate());
        System.out.println("SMA:"+iniOrbit.getA()/1000.+" km");

        // PROPAGATION
        final SpacecraftState sc = test.propagateInSlaveMode();
       
        System.out.println("Final orbit:");
        System.out.println("Date:"+sc.getDate());
        System.out.println("SMA:"+sc.getA()/1000.+" km");

    }
    
}
